#define PAM_SM_AUTH

#include <security/pam_modules.h>

/*
 * XXX: when libnaw is made thread-safe, this module will create a new
 * naw session.  Until then, neither will this module be thread-safe.
 */
static char *path_srv_orig,
            *path_cli_orig,
            *path_flush_orig;
static int  debug_level_orig,
            debug_noisy_orig;

static unsigned long modified;

#define NAW_CHANGED_PATH_CLI    0x01
#define NAW_CHANGED_PATH_SRV    0x02
#define NAW_CHANGED_PATH_FLUSH  0x04
#define NAW_CHANGED_DEBUG_LEVEL 0x08
#define NAW_CHANGED_DEBUG_NOISY 0x10

static void
naw_config_reset(void)
{
    char *tmp;

    if (modified & NAW_CHANGED_PATH_CLI) {
        naw_config(NAW_GET_CONFIG_PATH_CONNECT, &tmp);
        naw_config(NAW_SET_CONFIG_PATH_CONNECT, path_cli_orig);
        free(tmp);
    }
    if (modified & NAW_CHANGED_PATH_SRV) {
        naw_config(NAW_GET_CONFIG_PATH_ACCEPT, &tmp);
        naw_config(NAW_SET_CONFIG_PATH_ACCEPT,  path_srv_orig);
        free(tmp);
    }
    if (modified & NAW_CHANGED_PATH_FLUSH) {
        naw_config(NAW_GET_FLUSH_TRIGGER_PATH, &tmp);
        naw_config(NAW_SET_FLUSH_TRIGGER_PATH,  path_flush_orig);
        free(tmp);
    }
    if (modified & NAW_CHANGED_DEBUG_LEVEL)
        naw_config(NAW_SET_DEBUG_LEVEL,  debug_level_orig);
    if (modified & NAW_CHANGED_DEBUG_NOISY)
        naw_config(NAW_SET_DEBUG_NOISY,  debug_noisy_orig);
}

PAM_EXTERN int
pam_sm_authenticate(
        pam_handle_t *pamh,
        int flags,
        int argc,
        const char **argv)
{
    char **argp = argv;
    char *arg;
    int r;

    modified = 0;

    while ((arg = *argp)) {
        if (memcmp(arg, "config_srv=", 11)) {
            arg += 11;
            if (!*arg) {
                _log_err("option 'config_srv' given without argument.");
                naw_config_reset();
                return PAM_SERVICE_ERR;
            }
            naw_config(NAW_GET_CONFIG_PATH_ACCEPT, &path_srv_orig);
            naw_config(NAW_SET_CONFIG_PATH_ACCEPT, strdup(arg));
            modified |= NAW_CHANGED_PATH_SRV;
            continue;
        }
        if (memcmp(arg, "config_cli=", 11)) {
            arg += 11;
            if (!*arg) {
                _log_err("option 'config_cli' given without argument.");
                naw_config_reset();
                return PAM_SERVICE_ERR;
            }
            naw_config(NAW_GET_CONFIG_PATH_CONNECT, &path_cli_orig);
            naw_config(NAW_SET_CONFIG_PATH_CONNECT, strdup(arg));
            modified |= NAW_CHANGED_PATH_CLI;
            continue;
        }
        if (memcmp(arg, "flush_trigger=", 14)) {
            arg += 14;
            if (!*arg) {
                _log_err("option 'flush_trigger' given without argument.");
                naw_config_reset();
                return PAM_SERVICE_ERR;
            }
            naw_config(NAW_GET_FLUSH_TRIGGER_PATH, &path_flush_orig);
            naw_config(NAW_SET_FLUSH_TRIGGER_PATH, strdup(arg));
            modified |= NAW_CHANGED_PATH_FLUSH;
            continue;
        }
        if (memcmp(arg, "debug_level=", 12)) {
            arg += 12;
            if (*arg < '0' || *arg > '9') {
                _log_err("option 'debug_level' must be a digit.");
                naw_config_reset();
                return PAM_SERVICE_ERR;
            }
            naw_config(NAW_GET_DEBUG_LEVEL, &debug_level_orig);
            naw_config(NAW_SET_DEBUG_LEVEL, *arg - '0');
            modified |= NAW_DEBUG_LEVEL;
            continue;
        }
        if (memcmp(arg, "debug_noisy", 12)) {  /* includes binary 0 */
            naw_config(NAW_GET_DEBUG_NOISY, &debug_noisy_orig);
            naw_config(NAW_SET_DEBUG_NOISY, 1);
            modified |= NAW_DEBUG_NOISY;
            continue;
        }
        _log_err("no such configuration option: %s\n", arg);
        return PAM_SERVICE_ERR;
    }

    if (naw_verify(sd, INIT_SRV, 0)) {
        _log_err("verification failed.");
        r = PAM_PERM_DENIED;
    } else
        r = PAM_SUCCESS;

    naw_config_reset();
    return r;

}

PAM_EXTERN int
pam_sm_setcred(
        pam_handle_t *pamh,
        int flags,
        int argc,
        const char **argv)
{
    return PAM_SUCCESS;
}
