#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <strings.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include "naw_api.h"

#define DEFAULT_PORT 2999

int sd;

#ifndef CLIENT
void bye(int sig) {
    printf ("caught signal %d.\n", sig);
    fflush(stdout);
    close(sd);
    exit(1);
}
#endif

int main(int argc, char **argv) {
    struct sockaddr_in addr;
    char *bind_addr = "127.0.0.1";
    int port = htons(DEFAULT_PORT);
    char **args = argv+1;
    int addrlen;
#ifndef CLIENT
    int sdnew;
# ifdef SO_REUSEPORT
    int sockopt;
# endif
#endif

    if (*args)
        port = htons(atoi(*args++));

    if (*args)
        bind_addr = *args;

    sd = socket (AF_INET, SOCK_STREAM, 6);

    if (sd == -1) {
        perror("socket");
        return 1;
    }

    bzero(&addr, sizeof(addr));
    addr.sin_port = port;
    addr.sin_family = AF_INET;
    if (bind_addr) {
        if (!inet_aton(bind_addr, &addr.sin_addr)) {
            fprintf(stderr, "invalid ip address\n");
            return 1;
        }
    } else
        addr.sin_addr.s_addr = INADDR_ANY;

    addrlen = sizeof(addr);

    /* optional */
    naw_config(NAW_SET_CONFIG_PATH_CONNECT, NAW_CONFIGDIR"/config_cli.cdb");
    naw_config(NAW_SET_CONFIG_PATH_ACCEPT,  NAW_CONFIGDIR"/config_srv.cdb");
    naw_config(NAW_SET_FLUSH_TRIGGER_PATH,  NAW_CONFIGDIR"/flush");
    naw_config(NAW_SET_DEBUG_LEVEL, 5);
    naw_config(NAW_SET_DEBUG_NOISY, 1);

#ifndef CLIENT
# ifdef SO_REUSEPORT
    sockopt = 1;
    if (setsockopt(sd, SOL_SOCKET, SO_REUSEPORT, (void *)&sockopt, 4)) {
        perror("setsockopt");
        return 1;
    }
# endif
    if (bind(sd, (struct sockaddr *)&addr, addrlen)) {
        perror("bind");
        return 1;
    }

    if (listen(sd, 5)) {
        perror("listen");
        return 1;
    }

    signal(SIGINT, bye);

    for (;;) {
        printf ("waiting for connection...\n");
        sdnew = naw_accept(sd, (struct sockaddr *)&addr, &addrlen);
        if (sdnew == -1) {
            perror ("naw_accept");
            continue;
        }

        printf ("+++ accept call succeeded.\n");
        close(sdnew);
    }
#else
    if (naw_connect (sd, (struct sockaddr *)&addr, addrlen) == -1) {
        perror("naw_connect");
        return 1;
    }
    close(sd);
#endif

    return 0;
}

