#include <unistd.h>
#include <string.h>
#include <lx_string.h>
#include <cdb.h>
#include "misc.h"
#include "naw_log.h"


static inline int modentry_parse (c, data, mdbset, vp, vlen)
    struct cdb *c;
    struct naw_data *data;
    struct naw_module_dbset *mdbset;
    char *vp;
    unsigned vlen;
{
    {
        char *datsp;
        ub2_t datslen;

        mdbset->howmany = *vp++; vlen--;
        mdbset->modrefp = (ub4_t *)vp;

        naw_logfmt (NAW_DEBUG3, "found %d modules", mdbset->howmany);

        datsp = vp + mdbset->howmany*4;
        datslen = *(ub2_t *)datsp;
        datsp += 2;

        if (mdbset->howmany * 4 + 2 + datslen > vlen) {
            NAW_LOG(NAW_NOTICE,LXM_q("config: module entry corrupt"));
            return NAW_ERR;
        }

        if (naw_data_parse(data, c, datsp, datslen)) {
            NAW_LOG(NAW_NOTICE,LXM_q("config: error parsing data"));
            return NAW_ERR;
        }
    }

    return NAW_OK;
}

int modentry_run (c, handle, data)
    struct cdb *c;
    struct naw_handle *handle;
    struct naw_data *data;
{
    struct naw_module_dbset mdbset;
    unsigned pos, len;
    int r;

    pos = cdb_datapos(c);
    len = cdb_datalen(c);

    if (len < 3) {
        naw_logfmt(NAW_NOTICE,
                "config: module set value is undersized (%d bytes)", len);
        return NAW_ERR;
    }

    {
        char v[len];

        if (cdb_read(c, v, len, pos) == -1) {
            NAW_LOG(NAW_NOTICE, LXM_q("config: cdb short read"));
            return NAW_ERR;
        }

        r = modentry_parse(c, data, &mdbset, v, len);
        if (r != NAW_OK) return NAW_ERR;

        while (mdbset.howmany) {
            r = submodule_next(handle, c, &mdbset, data);
            switch (r) {
                case NAW_YES:
                    continue;
                default:
                    NAW_LOG(NAW_INFO,LXM_q("invalid return code from module"));
                case NAW_ERR:
                case NAW_NO:
                    return r;
            }
        }
    }
    return 0; /* quell warning */
}

int submodule (handle, c, ref, data)
    struct naw_handle *handle;
    struct cdb *c;
    ub4_t ref;
    struct naw_data *data;
{
    unsigned pos, len;
    char key[5];

    key_load(key, '.', ref);

    naw_logfmt(NAW_DEBUG3, "config: module ref 0x%x", ref);
    switch (cdb_find(c, key, 5)) {
        case -1:
            NAW_LOG(NAW_NOTICE,
                    LXM_q("config: cdb short read"));
            return NAW_ERR;
        case 0:
            NAW_LOG(NAW_NOTICE,
                    LXM_q("config: associated module name does not exist"));
            return NAW_ERR;
        default:
            NAW_LOG(NAW_DEBUG3,
                    LXM_q("config: found associated module name"));
    }

    pos = cdb_datapos(c);
    len = cdb_datalen(c);

    {
        char v[len+1];

        if (cdb_read(c, v, len, pos) == -1) {
            NAW_LOG(NAW_NOTICE,LXM_q("config: cdb short read"));
            return NAW_ERR;
        }

        v[len] = 0;
        return module_call(c, v, len, handle, data);
    }
}

static inline int modentry_globals_run(c, data)
    struct cdb *c;
    struct naw_data *data;
{
    static struct naw_handle handle = { 0, INIT_NONE, 0, 0 };
    int r;

    NAW_LOG(NAW_DEBUG3,LXM_q("config: found global module set"));

    if (naw_data_init(data)) return NAW_ERR;

    r = modentry_run(c, &handle, data);
    if (r != NAW_OK) return NAW_ERR;
    return NAW_OK;

    return 0;
}

static inline int modentry_globals_body(c, cf, data)
    struct cdb      *c;
    struct cdb_find *cf;
    struct naw_data *data;
{
    int v, r=0;
    ub1_t once=0;

    for (;;) {
        v = cdb_findnext(cf);
        switch (v) {
            case 1: once = 1;
                    r = modentry_globals_run(c, data);
                    break;
            case 0: if (once) return r;
                    NAW_LOG(NAW_DEBUG2,
                            LXM_q("global module set does not exist"));
                    return 0;
                    /* XXX ^-if this is to be consolidated, this should have
                     * its own return value. */
            case -1:NAW_LOG(NAW_NOTICE, LXM_q("config: cdb short read"));
                    return -1;
        }

        naw_data_free(data);
        data->bs = 0;

        if (r != NAW_OK) return r;
    }

    return r;
}

int modentry_globals(c)
    struct cdb *c;
{
    char key[] = { '=', 0, 0, 0, 0 };
    struct naw_data data;
    struct cdb_find cf;
    int r;

    if (cdb_findinit(&cf, c, key, 5) < 0) {
        NAW_LOG(NAW_NOTICE,LXM_q("config: error initializing cdb"));
        return -1;
    }

    data.bs = 0;
    r = modentry_globals_body(c, &cf, &data);
    if (data.bs) naw_data_free(&data);
    return r;
}
