/* include after: <sys/types.h> <sys/socket.h> */

#ifndef _NAW_API_H
#define _NAW_API_H

/***************************************************************************
  naw_connect

  use instead of the connect(2) call;
  semantics are the same.
 ***************************************************************************/
extern int  naw_connect(int, struct sockaddr *, int);

/***************************************************************************
  naw_accept

  use instead of the accept(2) call;
  semantics are the same.
 ***************************************************************************/
extern int  naw_accept (int, struct sockaddr *, int *);

/***************************************************************************
  naw_verify
    int   sd;
    ub1_t type;
    void *addr;

  use on a socket descriptor after the connect(2) or accept(2) call

  [sd] is the socket descriptor associated with the connection.

  [type] is either INIT_CLI or INIT_SRV, depending on whether connect(2) or
  accept(2) was called, respectively.

  [addr] is the sockaddr structure as passed to connect(2) or returned by
  accept(2).  May be null, in which case the information will be gathered
  internally.

  returns [sd] on success
  returns -1 on error.
 ***************************************************************************/
extern int  naw_verify (int, unsigned char, void *);

/***************************************************************************
  naw_config
    int type;
    (type) arg;

  implement configuration options as desired.
 ***************************************************************************/
/* type may be: */
#define NAW_SET_RESERVED            0  /* reserved: do not use */
#define NAW_SET_CONFIG_PATH_CONNECT 10 /* arg: [const char  *path ] */
#define NAW_GET_CONFIG_PATH_CONNECT 11 /* arg: [      char **path ] */
#define NAW_SET_CONFIG_PATH_ACCEPT  20 /* arg: [const char  *path ] */
#define NAW_GET_CONFIG_PATH_ACCEPT  21 /* arg: [      char **path ] */
#define NAW_SET_DEBUG_LEVEL         30 /* arg: [      int    level] */
#define NAW_GET_DEBUG_LEVEL         31 /* arg: [      int   *level] */
#define NAW_SET_DEBUG_NOISY         32 /* arg: [      int    level] */
#define NAW_GET_DEBUG_NOISY         33 /* arg: [      int   *level] */
#define NAW_SET_FLUSH_TRIGGER_PATH  40 /* arg: [const char  *path ] */
#define NAW_GET_FLUSH_TRIGGER_PATH  41 /* arg: [      char **path ] */
extern void naw_config (int type, ...);

#endif /* _NAW_API_H */
