#include <stdlib.h>
#include <string.h>
#include "lx_string.h"
#include "cfg_stuff.h"
#include "genlink.h"
#include "func.h"


lx_s  cfgtxt  = {0};
lx_s  cfgword = {0};
lx_gd gd_cfg;

unsigned int linenum = 0;
unsigned char eof = 0;

/***************************************************************************
  get_line_data (lx_gd *gd, lx_s *txt)

  Fills txt with another line of input from gd.  Skips comments and blank
  lines.  Dies on lines longer than 512 bytes.

  Returns 0 on success, or 1 on EOF.
 ***************************************************************************/
int get_line_data (gd, txt)
    lx_gd *gd;
    lx_s  *txt;
{
    char match;
    lx_s buf = {0};

    for (;;) {
        linenum++;
        if (lx_getln (&buf, gd, &match, 512))
            die_cfg (linenum, "lx_getln failed");

        if ( (match & MATCH_TOOLONG))
            die_cfg_linetoolong (linenum, 512);
        if (!(match & MATCH_OK)) eof++;

        lx_chomp_ws  (&buf);
        lx_chompf_ws (&buf);

        if (!buf.len) goto END;

        /* remove comments */
        if (buf.s[0] == '#') {
            buf.len = 0;
            goto END;
        }

        if (!txt->s) { if (lx_strcopy (txt, &buf)) die_nomem(); }
        else         { if (lx_strcat  (txt, &buf)) die_nomem(); }
        break;

END:
        if (eof) {
	    if (buf.s) lx_free (&buf);
            return 1;
        }
    }
    lx_free (&buf);
    return 0;
}


/***************************************************************************
  cfg_compare (lx_s *word, wordset *ws)


  Returns matching wordset on success, or 0 if no match was found.
 ***************************************************************************/

static inline wordset *cfg_compare (word, ws)
    lx_s *word;
    wordset *ws;
{
    for (;; ws++) {
        if (!ws->word) return 0;

        if (!lx_strscmp(word, ws->word))
            return (ws);
    }
}


/***************************************************************************
  cfg_length_word (lx_gd *gd, lx_s *txt)

  Gets the length of the next word in txt (words are delimited by
  whitespace).  If there is preceding whitespace, it will be removed.  If
  the line ends with a backslash, another line is read into it.  A backslash
  present at EOF is an error.

  Returns the word length, or -1 if no more words are on the line.
 ***************************************************************************/

int cfg_length_word (gd, txt)
    lx_gd *gd;
    lx_s  *txt;
{
    int wordlen;

    for (;;) {
        if (lx_chompf_ws(txt)) die_nomem();
        wordlen = lx_strsindex (txt, " \t", 2, 1);

	if (wordlen != -1) break;

        /* check for backslash at end of line */
        if (txt->len && (txt->s[txt->len-1] == '\\')) {
            /* txt->s[txt->len-1] = ' '; */
            txt->len--;
            if (get_line_data(gd, txt))
                die_cfg_syntax(linenum, "(EOF)", 5);
            continue;
        } else if (txt->len) return (txt->len);
          else break;

    }

    return wordlen;
}

/***************************************************************************
  cfg_check_word (lx_gd *gd, lx_s *txt, wordset *ws)


  Returns matching wordset on success, or 0 if no match was found.
 ***************************************************************************/
wordset *cfg_check_word (gd, txt, ws, wordlen)
    lx_gd *gd;
    lx_s  *txt;
    wordset *ws;
    int *wordlen;
{
    lx_s wd;  /* set explicitly */

    for (;;) {
        *wordlen = cfg_length_word (gd, txt);
        if (*wordlen == -1) break;

        wd.alloc = wd.len = (unsigned int) *wordlen;
        wd.s = txt->s;

        if (cfg_var_expand(&wd, txt)) {
            continue;
        }

        return (cfg_compare(&wd, ws));
    }

    return 0;
}

struct cfglink  *cfg_link_head = 0,
    	        *cfg_link_current = 0;
void cfg_parse (gd)
    lx_gd *gd;
{
    struct cfglink *link = 0;

    do {
        if (get_line_data (gd, &cfgtxt))
            continue;

        if (!link) GENLINK_new(link, struct cfglink);

        cfg_set_process(gd, &cfgtxt, link);

        if (link->action) {
            GENLINK_add(&cfg_link_current, link, cfg_next, struct cfglink);
	    if (!cfg_link_head)
                cfg_link_head = cfg_link_current;
            link = 0;
        }

    } while (!eof);

    if (link) GENLINK_destroy(link);
}
