#include <stdlib.h>
#include <string.h>
#include "misc.h"
#include "genlink.h"
#include "cfg_stuff.h"

struct varlink {
    lx_s key;
    lx_s value;
    struct varlink *next;
};

struct varlink *varlink_list = 0;
struct varlink *varlink_pos  = 0; /* keeps the last link to save time */


/***************************************************************************
  cfg_var (struct cfg_set_args *args)

  Grabs the key off the line, and places it into vl->key.
  Then grabs all remaining words off the line, and places them into
  vl->value.

  Returns zero.
 ***************************************************************************/

int
cfg_var (args)
    struct cfg_set_args *args;
{
    struct varlink *vl;
    lx_s tmp = {0};

    GENLINK_new(vl, struct varlink);

    vl->key.s   = 0;
    vl->value.s = 0;

    cfg_get_word(args->gd, &vl->key, args->txt);
    if (!vl->key.s) die_cfg_syntax(linenum, "(EOL)", 5);

    for (;;) {
        cfg_get_word(args->gd, &tmp, args->txt);
        if (!tmp.len) break;

        cfg_add_word(&vl->value, &tmp);
    }

    if (varlink_pos)
        GENLINK_add(&varlink_pos, vl, next, struct varlink);
    else
        GENLINK_add(&varlink_list, vl, next, struct varlink);

    varlink_pos = vl;

    if (tmp.s) lx_free (&tmp);

    return 0;
}


/***************************************************************************
  cfg_var_match (lx_s *var)

  Searches the variable list for var, and returns a pointer to the value.
  The returned value points directly into the linked list.

  If var exists, returns an lx_s pointer to the value.
  Otherwise, returns zero.
 ***************************************************************************/

static inline lx_s *cfg_var_match (var)
    lx_s *var;
{
    struct varlink *vl = varlink_list;

    while (vl) {
	if (lx_strcmp (&vl->key, var)) return &vl->value;
	vl = vl->next;
    }

    return 0;
}


/***************************************************************************
  cfg_var_expand (lx_s *var, lx_s *txt)

  Checks the variable list for var; if found, replaces var in txt.
  XXX: Should ensure that txt does not grow past a certain size.

  Returns 0 if no variable was replaced, or 1 if one was.
 ***************************************************************************/
int cfg_var_expand (var, txt)
    const lx_s *var;
    lx_s *txt;
{
    lx_s *tmp;

    if ((tmp = cfg_var_match(var))) {
	if (lx_strinsert (txt, tmp->s, tmp->len, 0, var->len))
	    die_nomem();

	return 1;
    }

    return 0;
}

